use std::str;

use crate::wav_writer::Note;

pub fn parse(source_string : String) -> Vec<Note>
{
    let mut note_vector : Vec<Note> = vec![];
    let mut octave      : i32 = 5;
    let mut length      : i32 = 4;

    let mut chars : str::Chars = source_string.chars();

    while let Some(character) = chars.next()
    {
        match character
        {
            'a'..='g' =>
            {
                let note : i32 = match character
                {
                    'c' => 0,
                    'd' => 2,
                    'e' => 4,
                    'f' => 5, 
                    'g' => 7,
                    'a' => 9,
                    'b' => 11,
                    _   => 0
                };

                let target_note : i32 = note + octave * 12;

                note_vector.push(Note { note : target_note, length });
            },
            'r' =>
            {
                note_vector.push(Note { note : -1, length });
            },
            'o' =>
            {
                let v : char = chars.next().expect("o 다음에 숫자를 지정해주시기 바랍니다.");
                let o : i32 = v as i32 - '0' as i32;

                if o >= 0 && o < 9
                {
                    octave = o;
                }
            },
            'l' =>
            {
                let v : char = chars.next().expect("l 다음에 숫자를 지정해주시기 바랍니다.");
                let l : i32 = v as i32 - '0' as i32;

                if l >= 1 && l <= 9
                {
                    length = l;
                }
            },
            _ => {},
        };
    }

    return note_vector;
}